@@ -1,10 +1,15 @@
-custom.measure <- function(X,fun=NULL,cond=NULL,verbose=TRUE,with.se=FALSE,...){
+custom.measure <- function(X, fun = NULL, cond = NULL, verbose = TRUE, with.se = FALSE, cilevel = 0.95, ...){
 if(is.null(fun)){stop("'fun' need to be specified")}
 if(is.function(fun)==FALSE){stop("'fun' needs to be a function")}
 if(is.null(X$simulated.data)){stop("Set 'ret=TRUE' in gformula() to work with custom measures.")}
 if(verbose==TRUE){if(X$setup$i.type=="custom"){cat("Note: you chose custom interventions. \n 'custom.measure' asssumes that the intervention values at the first time point (time=1) are not identical.\n\n")}}
 if(X$setup$measure=="custom"){stop("You can't apply 'custom measure' on an X onto which you already applied 'custom measure'.")}
 if(X$setup$B==0 & with.se==T){stop("Standard error can only be calculated if B>0")}
+if (!(is.numeric(cilevel) && length(cilevel) == 1 && cilevel > 0 && cilevel < 1)) {
+  stop("'cilevel' must be a single numeric value between 0 and 1.")
+}
+lcl <- (1 - cilevel) / 2
+ucl <- 1 - lcl
 if(X$setup$B>0){if(X$setup$B!=(length(X$simulated.data)-1)){X$setup$B<- (length(X$simulated.data)-1) }}
 sdata <- X$simulated.data
 odata <- X$observed.data
@@ -29,7 +34,7 @@ results.table[results.table$a1==X$setup$abar[i],"psi"] <- apply(as.matrix(sdata[
 if(X$setup$B>0){
 sbdata <- lapply(sdata, '[[', i); bfs <- matrix(NA,ncol=length(X$setup$Ynodes),nrow=X$setup$B)
 for(b in 2:(X$setup$B+1)){bfs[b-1,] <- apply(data.matrix(sbdata[[b]][sel,X$setup$Ynodes]),2,fun,...) } 
-results.table[results.table$a1==X$setup$abar[i],c("l95","u95")] <- t(apply(bfs,2,quantile,probs=c(0.025,0.975)))
+results.table[results.table$a1==X$setup$abar[i],c("lcl","ucl")] <- t(apply(bfs,2,quantile,probs = c(lcl, ucl)))
 if(with.se==TRUE){
   if(i==1){results.table$se<-NA}
   results.table[results.table$a1==X$setup$abar[i],c("se")] <- apply(bfs,2,sd)} 
@@ -43,7 +48,7 @@ results.table[results.table$a1==X$setup$abar[i,1],"psi"] <- apply(as.matrix(sdat
 if(X$setup$B>0){
 sbdata <- lapply(sdata, '[[', i); bfs <- matrix(NA,ncol=length(X$setup$Ynodes),nrow=X$setup$B)
 for(b in 2:(X$setup$B+1)){bfs[b-1,] <- apply(data.matrix(sbdata[[b]][sel,X$setup$Ynodes]),2,fun,...) } 
-results.table[results.table$a1==X$setup$abar[i],c("l95","u95")] <- t(apply(bfs,2,quantile,probs=c(0.025,0.975)))
+results.table[results.table$a1==X$setup$abar[i],c("lcl","ucl")] <- t(apply(bfs,2,quantile,probs = c(lcl, ucl)))
 if(with.se==TRUE){
   if(i==1){results.table$se<-NA}
   results.table[results.table$a1==X$setup$abar[i],c("se")] <- apply(bfs,2,sd)}  
@@ -96,9 +101,9 @@ for(b in 2:(X$setup$B+1)){
     ofs[b-1,] <- apply(data.matrix( odata[[b]][sel,c(X$setup$Ynodes,X$setup$Lnodes)]),2,fun,...) 
 }
 dfs <- bfs-ofs
-results.table[results.table$a1=="natural",c("l95","u95")] <- t(apply(subset(bfs,select=X$setup$Ynodes),2,quantile,probs=c(0.025,0.975)))
-results.table[results.table$a1=="observed",c("l95","u95")] <- t(apply(subset(ofs,select=X$setup$Ynodes),2,quantile,probs=c(0.025,0.975)))
-results.table[results.table$a1=="difference",c("l95","u95")] <-    t(apply(subset(dfs,select=X$setup$Ynodes),2,quantile,probs=c(0.025,0.975)))
+results.table[results.table$a1=="natural",c("lcl","ucl")] <- t(apply(subset(bfs,select=X$setup$Ynodes),2,quantile,probs = c(lcl, ucl)))
+results.table[results.table$a1=="observed",c("lcl","ucl")] <- t(apply(subset(ofs,select=X$setup$Ynodes),2,quantile,probs = c(lcl, ucl)))
+results.table[results.table$a1=="difference",c("lcl","ucl")] <- t(apply(subset(dfs,select=X$setup$Ynodes),2,quantile,probs = c(lcl, ucl)))
   if(with.se==TRUE){
   results.table$se<-NA
   results.table[results.table$a1=="natural",c("se")] <- apply(subset(bfs,select=X$setup$Ynodes),2,sd)
@@ -107,46 +112,46 @@ results.table[results.table$a1=="difference",c("l95","u95")] <-    t(apply(subse
   }
   if(is.null(X$setup$Lnodes)==FALSE){
     if(is.wholenumber(length(X$setup$Lnodes)/length(X$setup$Ynodes)) | X$setup$n.t==1){
-    results.table[results.table$a1=="natural",grep(":l95",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
-    results.table[results.table$a1=="natural",grep(":u95",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
-    results.table[results.table$a1=="observed",grep(":l95",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
-    results.table[results.table$a1=="observed",grep(":u95",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
-    results.table[results.table$a1=="difference",grep(":l95",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
-    results.table[results.table$a1=="difference",grep(":u95",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
+    results.table[results.table$a1=="natural",grep(":lcl",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
+    results.table[results.table$a1=="natural",grep(":ucl",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
+    results.table[results.table$a1=="observed",grep(":lcl",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
+    results.table[results.table$a1=="observed",grep(":ucl",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
+    results.table[results.table$a1=="difference",grep(":lcl",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
+    results.table[results.table$a1=="difference",grep(":ucl",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=length(X$setup$Lnodes)/X$setup$n.t,byrow=T)
         if(with.se==TRUE){
             results.table[results.table$a1=="natural",paste0("L_",1:(length(X$setup$Lnodes)/X$setup$n.t),":se")]    <-  matrix(apply(bfs[,X$setup$Lnodes],2,sd),ncol=(length(X$setup$Lnodes)/X$setup$n.t),byrow=T)
             results.table[results.table$a1=="observed",paste0("L_",1:(length(X$setup$Lnodes)/X$setup$n.t),":se")]   <-  matrix(apply(ofs[,X$setup$Lnodes],2,sd),ncol=(length(X$setup$Lnodes)/X$setup$n.t),byrow=T)
             results.table[results.table$a1=="difference",paste0("L_",1:(length(X$setup$Lnodes)/X$setup$n.t),":se")] <-  matrix(apply(dfs[,X$setup$Lnodes],2,sd),ncol=(length(X$setup$Lnodes)/X$setup$n.t),byrow=T)}
     }else{
       if(length(lblocks)>1 & length(unique(lblocks[-1]))==1){
-        results.table[results.table$a1=="natural" & results.table$time!=1,grep(":l95",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=max(lblocks),byrow=T)
-        results.table[results.table$a1=="natural" & results.table$time!=1,grep(":u95",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=max(lblocks),byrow=T)
-        results.table[results.table$a1=="observed" & results.table$time!=1,grep(":l95",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=max(lblocks),byrow=T)
-        results.table[results.table$a1=="observed" & results.table$time!=1,grep(":u95",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=max(lblocks),byrow=T)
-        results.table[results.table$a1=="difference" & results.table$time!=1,grep(":l95",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=max(lblocks),byrow=T)
-        results.table[results.table$a1=="difference" & results.table$time!=1,grep(":u95",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=max(lblocks),byrow=T)
+        results.table[results.table$a1=="natural" & results.table$time!=1,grep(":lcl",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=max(lblocks),byrow=T)
+        results.table[results.table$a1=="natural" & results.table$time!=1,grep(":ucl",colnames(results.table))] <-  matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=max(lblocks),byrow=T)
+        results.table[results.table$a1=="observed" & results.table$time!=1,grep(":lcl",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=max(lblocks),byrow=T)
+        results.table[results.table$a1=="observed" & results.table$time!=1,grep(":ucl",colnames(results.table))] <-  matrix(apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=max(lblocks),byrow=T)
+        results.table[results.table$a1=="difference" & results.table$time!=1,grep(":lcl",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=max(lblocks),byrow=T)
+        results.table[results.table$a1=="difference" & results.table$time!=1,grep(":ucl",colnames(results.table))] <-  matrix(apply(dfs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=max(lblocks),byrow=T)
           if(with.se==TRUE){
             results.table[results.table$a1=="natural" & results.table$time!=1,paste0("L_",1:max(lblocks),":se")]      <-  matrix(apply(bfs[,X$setup$Lnodes],2,sd),ncol=max(lblocks),byrow=T)
             results.table[results.table$a1=="observed" & results.table$time!=1,paste0("L_",1:max(lblocks),":se")]     <-  matrix(apply(ofs[,X$setup$Lnodes],2,sd),ncol=max(lblocks),byrow=T)
             results.table[results.table$a1=="difference" & results.table$time!=1,paste0("L_",1:max(lblocks),":se")]   <-  matrix(apply(dfs[,X$setup$Lnodes],2,sd),ncol=max(lblocks),byrow=T)}
       }else{
         if(max(lblocks,na.rm=T)==1){
-          results.table[results.table$a1=="natural", paste(paste0("L_",1),"l95",sep=":")] <-
-          matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(0.025)),ncol=1,byrow=T)[updat.index]
-          results.table[results.table$a1=="natural", paste(paste0("L_",1),"u95",sep=":")] <-
-          matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(0.975)),ncol=1,byrow=T)[updat.index]
-          results.table[results.table$a1=="observed", c(paste(paste0("L_",1),"l95",sep=":"))] <-
-          apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(0.025))[updat.index]
-          results.table[results.table$a1=="observed", c(paste(paste0("L_",1),"u95",sep=":"))] <-
-          apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(0.975))[updat.index]
-          col.order <- c(1:grep("psi",colnames(results.table)),which(colnames(results.table)%in%c("l95","u95")))
+          results.table[results.table$a1=="natural", paste(paste0("L_",1),"lcl",sep=":")] <-
+          matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(lcl)),ncol=1,byrow=T)[updat.index]
+          results.table[results.table$a1=="natural", paste(paste0("L_",1),"ucl",sep=":")] <-
+          matrix(apply(bfs[,X$setup$Lnodes],2,quantile,probs=c(ucl)),ncol=1,byrow=T)[updat.index]
+          results.table[results.table$a1=="observed", c(paste(paste0("L_",1),"lcl",sep=":"))] <-
+          apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(lcl))[updat.index]
+          results.table[results.table$a1=="observed", c(paste(paste0("L_",1),"ucl",sep=":"))] <-
+          apply(ofs[,X$setup$Lnodes],2,quantile,probs=c(ucl))[updat.index]
+          col.order <- c(1:grep("psi",colnames(results.table)),which(colnames(results.table)%in%c("lcl","ucl")))
           col.order <- c(col.order,sort(grep(paste0("L_",1),colnames(results.table))))
           if(with.se==TRUE){col.order<-c(col.order, which(colnames(results.table)%in%c("se")))}
           results.table <- results.table[,col.order]
-          results.table[results.table$a1=="difference", paste(paste0("L_",1),"l95",sep=":")] <-
-          apply(bfs[,X$setup$Lnodes]-ofs[,X$setup$Lnodes],2,quantile,probs=c(0.025))[updat.index]
-          results.table[results.table$a1=="difference", paste(paste0("L_",1),"u95",sep=":")] <-
-          apply(bfs[,X$setup$Lnodes]-ofs[,X$setup$Lnodes],2,quantile,probs=c(0.975))[updat.index]
+          results.table[results.table$a1=="difference", paste(paste0("L_",1),"lcl",sep=":")] <-
+          apply(bfs[,X$setup$Lnodes]-ofs[,X$setup$Lnodes],2,quantile,probs=c(lcl))[updat.index]
+          results.table[results.table$a1=="difference", paste(paste0("L_",1),"ucl",sep=":")] <-
+          apply(bfs[,X$setup$Lnodes]-ofs[,X$setup$Lnodes],2,quantile,probs=c(ucl))[updat.index]
           if(with.se==TRUE){
             results.table[results.table$a1=="natural",paste0("L_",1,":se")]      <-  matrix(apply(bfs[,X$setup$Lnodes],2,sd),ncol=1,byrow=T)[updat.index]
             results.table[results.table$a1=="observed",paste0("L_",1,":se")]     <-  matrix(apply(ofs[,X$setup$Lnodes],2,sd),ncol=1,byrow=T)[updat.index]
