@@ -2,8 +2,9 @@
 # 0) Wrapper function: as.character/as.numeric
 # 1) Check -> Note: you chose custom interventions. 
 # 'custom.measure' asssumes that the intervention values at the first time point (time=1) are not identical.
+# 2) examplescrosssec.r -> teilweise gibt das objekt nicht simulated.data etc. zurueck!  
+# 3) MSM: weniger abar sollte moeglichs ein: anstatt do.call(...) ueber alle simulierten Datensaetze anzuwenden, wuerde man z.B. X$simulated.data[[1]][[c(3, 4, 5)]]
 # 4) maybe test mixed interventions?
-# 5) contrast(values=c(0,1), fun="ratio") function -> maybe using custom.estimand?
 # 7) natural bounds 
 # 8) shift 
 # 9) SACE ; and competing() and revisit definitions of cbar
@@ -13,7 +14,8 @@ gformula <- function(X, Anodes, Ynodes, Lnodes = NULL, Cnodes = NULL,
                         abar =  NULL, cbar = "uncensored", survivalY = FALSE,
                         Yform = "GLM", Lform = "GLM", Aform = "GLM", Cform="GLM",
                         calc.support = FALSE, B = 0, ret=FALSE,  
-                        ncores=1, verbose=TRUE, seed=NULL, prog=NULL,...){
+                        ncores=1, verbose=TRUE, seed=NULL, prog=NULL,
+                        cilevel = 0.95, ...){
 
 ### checks and setup ###
 if(is.null(prog)==FALSE){write(matrix("started with setup..."),file=paste(prog,"/progress.txt",sep=""))}
@@ -83,6 +85,11 @@ if(any(substr(model.families[which(colnames(X)%in%Anodes)],1,4)=="mult")){
       }}else{if(verbose==TRUE){cat("Note: No control check for individual interventions on factor variables are done \n Make sure your setup is correct. \n ")}}
   }else{catint<-FALSE}
 
+if (!(is.numeric(cilevel) && length(cilevel) == 1 && cilevel > 0 && cilevel < 1)) {
+  stop("'cilevel' must be a single numeric value between 0 and 1.")
+}
+lcl <- (1 - cilevel) / 2
+ucl <- 1 - lcl
 
 if(verbose==TRUE){
 if(any(abar=="natural")==FALSE){rel.nodes<-c(Ynodes,Lnodes)}else{rel.nodes<-c(Ynodes,Lnodes,Cnodes,Anodes)}
@@ -390,62 +397,62 @@ if(B>0){
                                   obs.data <- obs.data[-c(2:(B+1))[unlist(boot.failure)]]; sim.data <- sim.data[-c(2:(B+1))[unlist(boot.failure)]]
       if(verbose==TRUE){cat(paste("Caution:",sum(unlist(boot.failure)),"bootstrap sample(s) were removed due to errors \n"))}   }
   newB <- B-sum(unlist(boot.failure))
-  if(i.type!="natural"){store.results[,c("l95","u95")] <-  t(apply(matrix(unlist(analysis.b),ncol=newB),1,quantile,probs=c(0.025,0.975)))}else{
+  if(i.type!="natural"){store.results[,c("lcl","ucl")] <-  t(apply(matrix(unlist(analysis.b),ncol=newB),1,quantile,probs=c(lcl,ucl)))}else{
   sim.results <- matrix(unlist(analysis.b),ncol=newB,dimnames=list(colnames(analysis.b[[1]]),NULL))
-  store.results[store.results$a1=="natural",c("l95","u95")]  <-  t(apply(subset(sim.results,subset=rownames(sim.results)%in%Ynodes),1,quantile,probs=c(0.025,0.975)))
-  store.results[store.results$a1=="observed",c("l95","u95")] <-  t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Ynodes)),ncol=newB),1,quantile,probs=c(0.025,0.975)))
-  store.results[store.results$a1=="difference",c("l95","u95")] <-  t(apply(sim.results[rownames(sim.results)%in%Ynodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Ynodes)),ncol=newB),1,quantile,probs=c(0.025,0.975)))
+  store.results[store.results$a1=="natural",c("lcl","ucl")]  <-  t(apply(subset(sim.results,subset=rownames(sim.results)%in%Ynodes),1,quantile,probs=c(lcl,ucl)))
+  store.results[store.results$a1=="observed",c("lcl","ucl")] <-  t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Ynodes)),ncol=newB),1,quantile,probs=c(lcl,ucl)))
+  store.results[store.results$a1=="difference",c("lcl","ucl")] <-  t(apply(sim.results[rownames(sim.results)%in%Ynodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Ynodes)),ncol=newB),1,quantile,probs=c(lcl,ucl)))
   if(is.null(Lnodes)==FALSE){
     if(n.t==1){
-    store.results[store.results$a1=="natural", paste(paste0("L_",1:(length(Lnodes)/n.t)),"l95",sep=":")] <- 
-    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(0.025)),ncol=(length(Lnodes)/n.t),byrow=T)
-    store.results[store.results$a1=="natural", paste(paste0("L_",1:(length(Lnodes)/n.t)),"u95",sep=":")] <- 
-    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(0.975)),ncol=(length(Lnodes)/n.t),byrow=T)
-    store.results[store.results$a1=="observed", c(paste(paste0("L_",1:(length(Lnodes)/n.t)),"l95",sep=":"))] <- 
-    matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.025))),ncol=(length(Lnodes)/n.t),byrow=TRUE)
-    store.results[store.results$a1=="observed", c(paste(paste0("L_",1:(length(Lnodes)/n.t)),"u95",sep=":"))] <- 
-    matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.975))),ncol=(length(Lnodes)/n.t),byrow=TRUE)
-    col.order <- c(1:grep("psi",colnames(store.results)),which(colnames(store.results)%in%c("l95","u95")))
+    store.results[store.results$a1=="natural", paste(paste0("L_",1:(length(Lnodes)/n.t)),"lcl",sep=":")] <- 
+    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(lcl)),ncol=(length(Lnodes)/n.t),byrow=T)
+    store.results[store.results$a1=="natural", paste(paste0("L_",1:(length(Lnodes)/n.t)),"ucl",sep=":")] <- 
+    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(ucl)),ncol=(length(Lnodes)/n.t),byrow=T)
+    store.results[store.results$a1=="observed", c(paste(paste0("L_",1:(length(Lnodes)/n.t)),"lcl",sep=":"))] <- 
+    matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(lcl))),ncol=(length(Lnodes)/n.t),byrow=TRUE)
+    store.results[store.results$a1=="observed", c(paste(paste0("L_",1:(length(Lnodes)/n.t)),"ucl",sep=":"))] <- 
+    matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(ucl))),ncol=(length(Lnodes)/n.t),byrow=TRUE)
+    col.order <- c(1:grep("psi",colnames(store.results)),which(colnames(store.results)%in%c("lcl","ucl")))
     for(i in 1:(length(Lnodes)/n.t)){col.order <- c(col.order,sort(grep(paste0("L_",i),colnames(store.results)))) }
     store.results <- store.results[,col.order]
-    store.results[store.results$a1=="difference", paste(paste0("L_",1:(length(Lnodes)/n.t)),"l95",sep=":")] <- 
-    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.025)),ncol=(length(Lnodes)/n.t),byrow=T)
-    store.results[store.results$a1=="difference", paste(paste0("L_",1:(length(Lnodes)/n.t)),"u95",sep=":")] <- 
-    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.975)),ncol=(length(Lnodes)/n.t),byrow=T)
+    store.results[store.results$a1=="difference", paste(paste0("L_",1:(length(Lnodes)/n.t)),"lcl",sep=":")] <- 
+    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(lcl)),ncol=(length(Lnodes)/n.t),byrow=T)
+    store.results[store.results$a1=="difference", paste(paste0("L_",1:(length(Lnodes)/n.t)),"ucl",sep=":")] <- 
+    matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(ucl)),ncol=(length(Lnodes)/n.t),byrow=T)
     }else{
       if(length(lblocks)>1 & length(unique(lblocks[-1]))==1 & lblocks[1]==0){
-      store.results[store.results$a1=="natural" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"l95",sep=":")] <- 
-      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(0.025)),ncol=max(lblocks),byrow=T)
-      store.results[store.results$a1=="natural" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"u95",sep=":")] <- 
-      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(0.975)),ncol=max(lblocks),byrow=T)
-      store.results[store.results$a1=="observed" & store.results$time!=1, c(paste(paste0("L_",1:max(lblocks)),"l95",sep=":"))] <- 
-      matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.025))),ncol=max(lblocks),byrow=TRUE)
-      store.results[store.results$a1=="observed" & store.results$time!=1, c(paste(paste0("L_",1:max(lblocks)),"u95",sep=":"))] <- 
-      matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.975))),ncol=max(lblocks),byrow=TRUE)
-      col.order <- c(1:grep("psi",colnames(store.results)),which(colnames(store.results)%in%c("l95","u95")))
+      store.results[store.results$a1=="natural" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"lcl",sep=":")] <- 
+      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(lcl)),ncol=max(lblocks),byrow=T)
+      store.results[store.results$a1=="natural" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"ucl",sep=":")] <- 
+      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(ucl)),ncol=max(lblocks),byrow=T)
+      store.results[store.results$a1=="observed" & store.results$time!=1, c(paste(paste0("L_",1:max(lblocks)),"lcl",sep=":"))] <- 
+      matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(lcl))),ncol=max(lblocks),byrow=TRUE)
+      store.results[store.results$a1=="observed" & store.results$time!=1, c(paste(paste0("L_",1:max(lblocks)),"ucl",sep=":"))] <- 
+      matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(ucl))),ncol=max(lblocks),byrow=TRUE)
+      col.order <- c(1:grep("psi",colnames(store.results)),which(colnames(store.results)%in%c("lcl","ucl")))
       for(i in 1:max(lblocks)){col.order <- c(col.order,sort(grep(paste0("L_",i),colnames(store.results)))) }
       store.results <- store.results[,col.order]
-      store.results[store.results$a1=="difference" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"l95",sep=":")] <- 
-      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.025)),ncol=max(lblocks),byrow=T)
-      store.results[store.results$a1=="difference" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"u95",sep=":")] <- 
-      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.975)),ncol=max(lblocks),byrow=T)
+      store.results[store.results$a1=="difference" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"lcl",sep=":")] <- 
+      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(lcl)),ncol=max(lblocks),byrow=T)
+      store.results[store.results$a1=="difference" & store.results$time!=1, paste(paste0("L_",1:max(lblocks)),"ucl",sep=":")] <- 
+      matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(ucl)),ncol=max(lblocks),byrow=T)
         }else{
           if(max(lblocks,na.rm=T)==1){
-          store.results[store.results$a1=="natural", paste(paste0("L_",1),"l95",sep=":")] <- 
-          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(0.025))[updat.index],ncol=1,byrow=T)
-          store.results[store.results$a1=="natural", paste(paste0("L_",1),"u95",sep=":")] <- 
-          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(0.975))[updat.index],ncol=1,byrow=T)
-          store.results[store.results$a1=="observed", c(paste(paste0("L_",1),"l95",sep=":"))] <- 
-          matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.025))),1,byrow=TRUE)[,updat.index]
-          store.results[store.results$a1=="observed", c(paste(paste0("L_",1),"u95",sep=":"))] <- 
-          matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.975))),1,byrow=TRUE)[,updat.index]
-          col.order <- c(1:grep("psi",colnames(store.results)),which(colnames(store.results)%in%c("l95","u95")))
+          store.results[store.results$a1=="natural", paste(paste0("L_",1),"lcl",sep=":")] <- 
+          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(lcl))[updat.index],ncol=1,byrow=T)
+          store.results[store.results$a1=="natural", paste(paste0("L_",1),"ucl",sep=":")] <- 
+          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F],1,quantile,probs=c(ucl))[updat.index],ncol=1,byrow=T)
+          store.results[store.results$a1=="observed", c(paste(paste0("L_",1),"lcl",sep=":"))] <- 
+          matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(lcl))),1,byrow=TRUE)[,updat.index]
+          store.results[store.results$a1=="observed", c(paste(paste0("L_",1),"ucl",sep=":"))] <- 
+          matrix(t(apply(matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(ucl))),1,byrow=TRUE)[,updat.index]
+          col.order <- c(1:grep("psi",colnames(store.results)),which(colnames(store.results)%in%c("lcl","ucl")))
           col.order <- c(col.order,sort(grep(paste0("L_",1),colnames(store.results))))
           store.results <- store.results[,col.order]
-          store.results[store.results$a1=="difference", paste(paste0("L_",1),"l95",sep=":")] <- 
-          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.025)),ncol=1,byrow=T)[updat.index,]
-          store.results[store.results$a1=="difference", paste(paste0("L_",1),"u95",sep=":")] <- 
-          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(0.975)),ncol=1,byrow=T)[updat.index,]
+          store.results[store.results$a1=="difference", paste(paste0("L_",1),"lcl",sep=":")] <- 
+          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(lcl)),ncol=1,byrow=T)[updat.index,]
+          store.results[store.results$a1=="difference", paste(paste0("L_",1),"ucl",sep=":")] <- 
+          matrix(apply(sim.results[rownames(sim.results)%in%Lnodes,,drop=F]-matrix(unlist(lapply(obs.data[-1],lrmean,ind=Lnodes)),ncol=newB),1,quantile,probs=c(ucl)),ncol=1,byrow=T)[updat.index,]
       }}
     }
   }}
