@@ -3,7 +3,7 @@ plot.gformula <- function(x, msm.method=c("line","loess","gam","none"),
                              cols=NULL, weight=NULL, xaxis=NULL,
                              variable="psi", difference=FALSE,  ...){
 
-a1<-psi<-l95<-u95<-relvar<-Strategy<-sel.l95<-sel.u95<-NULL
+a1<-psi<-lcl<-ucl<-relvar<-Strategy<-sel.lcl<-sel.ucl<-NULL
 gobject <- x
 msm.method  <- match.arg(msm.method)
 if(msm.method=="none" & is.null(weight)==FALSE){stop("For weighted MSM estimation, `msm.method' cannot be `none'")}
@@ -37,8 +37,8 @@ if(msm.method=="none"){gg2<-gg1}
 if(msm.method=="line" & bin.int==FALSE){gg2<-gg1+ geom_line(linetype=3, linewidth=1.05)}
 if(msm.method=="loess" | msm.method=="gam"){mymethod=paste("(estimated with ",msm.method,")",sep="")
                         gg2<- gg1+ geom_smooth(method = msm.method, se = FALSE, aes(weight=weight))}else{mymethod=NULL}
-if(CI==TRUE & bin.int==FALSE){gg2 <- gg2 + geom_ribbon(aes(ymin = l95, ymax = u95, fill=as.factor(time)), alpha=0.1,show.legend = FALSE)}
-if(CI==TRUE & bin.int==TRUE & n.t==1){gg2 <- gg2 + geom_pointrange(aes(ymin = l95, ymax = u95))}
+if(CI==TRUE & bin.int==FALSE){gg2 <- gg2 + geom_ribbon(aes(ymin = lcl, ymax = ucl, fill=as.factor(time)), alpha=0.1,show.legend = FALSE)}
+if(CI==TRUE & bin.int==TRUE & n.t==1){gg2 <- gg2 + geom_pointrange(aes(ymin = lcl, ymax = ucl))}
 gg3 <- gg2 + scale_color_manual(values = cols) + scale_fill_manual(values = cols) +
        scale_x_continuous("Intervention", breaks=round(c(seq(min(results$a1),max(results$a1),length.out=length(results$a1[results$time==min(results$time)]))),digits=1) ) +
        scale_y_continuous(expression(psi)) +
@@ -61,15 +61,15 @@ if(is.null(cols)==FALSE){if(length(cols)!=length(results$time[results$time==1]))
 if(is.null(cols)){cols<-mycolors[1:length(results$time[results$time==1])]}
 results$Strategy <- rep(c("natural \n intervention", "observed \n data"),length(unique(results$time)))
 colnames(results)[colnames(results)==variable] <- "relvar"
-if(variable=="psi"){colnames(results)[colnames(results)%in%c("l95","u95")]<-c("sel.l95","sel.u95")}else{
-                    colnames(results)[grep(paste0(variable,":"),colnames(results))]<-c("sel.l95","sel.u95")}
+if(variable=="psi"){colnames(results)[colnames(results)%in%c("lcl","ucl")]<-c("sel.lcl","sel.ucl")}else{
+                    colnames(results)[grep(paste0(variable,":"),colnames(results))]<-c("sel.lcl","sel.ucl")}
 
 gg1 <- ggplot(results, aes(x=time,y=relvar,col=as.factor(Strategy)))  + geom_point(size=1.75, position = position_dodge(width = 0+0.25*(CI==T & msm.method=="none"))) + theme_bw()
 if(msm.method=="none"){gg2<-gg1}
 if(msm.method!="none"){gg2<-gg1+ geom_line(linetype=3, linewidth=1.05)}
 if(msm.method=="loess" | msm.method=="gam"){cat("Note: MSM methods 'loess' and 'gam' not supported for natural interventions. \n Too few observations available for modeling (typically). \n msm.method='line' is used. \n")}
-if(CI==TRUE & msm.method!="none"){gg2 <- gg2 + geom_ribbon(aes(ymin = sel.l95, ymax = sel.u95, fill=as.factor(Strategy)), alpha=0.1,show.legend = FALSE)}
-if(CI==TRUE & msm.method=="none"){gg2 <- gg2 + geom_pointrange(aes(ymin = sel.l95, ymax = sel.u95), linewidth=1.05, show.legend=FALSE, position = position_dodge(width = 0.25))}
+if(CI==TRUE & msm.method!="none"){gg2 <- gg2 + geom_ribbon(aes(ymin = sel.lcl, ymax = sel.ucl, fill=as.factor(Strategy)), alpha=0.1,show.legend = FALSE)}
+if(CI==TRUE & msm.method=="none"){gg2 <- gg2 + geom_pointrange(aes(ymin = sel.lcl, ymax = sel.ucl), linewidth=1.05, show.legend=FALSE, position = position_dodge(width = 0.25))}
 gg3 <- gg2 + scale_color_manual(values = cols) + scale_fill_manual(values = cols) +
        scale_x_continuous("Time",breaks=unique(results$time)) +
        scale_y_continuous(label) +
@@ -82,15 +82,15 @@ gg3 <- gg2 + scale_color_manual(values = cols) + scale_fill_manual(values = cols
 if(variable=="psi"){label<-expression(psi)}else{label<-expression(variable)}
 colnames(results)[colnames(results)==variable] <- "relvar"
 diff.results <- results[results$a1=="difference",]
-if(variable=="psi"){colnames(diff.results)[colnames(diff.results)%in%c("l95","u95")]<-c("sel.l95","sel.u95")}else{
-                    colnames(diff.results)[grep(paste0(variable,":"),colnames(diff.results))]<-c("sel.l95","sel.u95")}
+if(variable=="psi"){colnames(diff.results)[colnames(diff.results)%in%c("lcl","ucl")]<-c("sel.lcl","sel.ucl")}else{
+                    colnames(diff.results)[grep(paste0(variable,":"),colnames(diff.results))]<-c("sel.lcl","sel.ucl")}
 if(is.null(cols)==FALSE){cols <- "black"}
 gg1 <- ggplot(diff.results, aes(x=time,y=relvar))  + geom_point(size=1.75, position = position_dodge(width = 0+0.3*(CI==T & msm.method=="none"))) + theme_bw()
 if(msm.method=="none"){gg2<-gg1}
 if(msm.method!="none"){gg2<-gg1+ geom_line(linetype=3, linewidth=1.05)}
 if(msm.method=="loess" | msm.method=="gam"){cat("Note: MSM methods 'loess' and 'gam' not supported for natural interventions. \n Too few observations available for modeling (typically). \n MSM method='line' is used. \n")}
-if(CI==TRUE & msm.method!="none"){gg2 <- gg2 + geom_ribbon(aes(ymin = sel.l95, ymax = sel.u95), alpha=0.1,show.legend = FALSE)}
-if(CI==TRUE & msm.method=="none"){gg2 <- gg2 + geom_pointrange(aes(ymin = sel.l95, ymax = sel.u95), linewidth=1.05, show.legend=FALSE, position = position_dodge(width = 0.25))}
+if(CI==TRUE & msm.method!="none"){gg2 <- gg2 + geom_ribbon(aes(ymin = sel.lcl, ymax = sel.ucl), alpha=0.1,show.legend = FALSE)}
+if(CI==TRUE & msm.method=="none"){gg2 <- gg2 + geom_pointrange(aes(ymin = sel.lcl, ymax = sel.ucl), linewidth=1.05, show.legend=FALSE, position = position_dodge(width = 0.25))}
 gg3 <- gg2 + scale_color_manual(values = cols) + scale_fill_manual(values = cols) +
        scale_x_continuous("Time",breaks=unique(results$time)) +
        scale_y_continuous("Difference") + geom_hline(yintercept = 0) +
@@ -113,8 +113,8 @@ gg1 <- ggplot(results, aes(x=time,y=psi,col=as.factor(Strategy), fill=as.factor(
 if(msm.method=="none"){gg2<-gg1}
 if(msm.method!="none"){gg2<-gg1+ geom_line(linetype=3, linewidth=1.05, show.legend = FALSE)}
 if(msm.method=="loess" | msm.method=="gam"){msm.method<-"line"; cat("Note: MSM methods 'loess' and 'gam' not supported for custom interventions (and survival setting). \n Too few observations available for modeling (typically). \n msm.method='line' is used. \n")}
-if(CI==TRUE & bin.int==FALSE & msm.method=="line"){gg2 <- gg2 + geom_ribbon(aes(ymin = l95, ymax = u95), alpha=0.1,show.legend = FALSE)}
-if(CI==TRUE & (bin.int==TRUE | msm.method=="none")){gg2 <- gg2 + geom_pointrange(aes(ymin = l95, ymax = u95), show.legend=FALSE, position = position_dodge(width = 0.3))}
+if(CI==TRUE & bin.int==FALSE & msm.method=="line"){gg2 <- gg2 + geom_ribbon(aes(ymin = lcl, ymax = ucl), alpha=0.1,show.legend = FALSE)}
+if(CI==TRUE & (bin.int==TRUE | msm.method=="none")){gg2 <- gg2 + geom_pointrange(aes(ymin = lcl, ymax = ucl), show.legend=FALSE, position = position_dodge(width = 0.3))}
 gg3 <- gg2 + scale_color_manual(values = cols) +  scale_fill_manual(values = cols) +
        scale_x_continuous("Time",breaks=unique(results$time)) +
        scale_y_continuous(expression(psi)) +
